/****
Approximate Class - 8/1/00 cmsief
This is an abstract base class for approximations
****/
#ifndef _MAPS_APPROX_
#define _MAPS_APPROX_

#include "maps_general.h"

/* The following definitions are used in
   SearchCriterion::ResetApproximation to allow user to
   reset to Null or Kriging. Only used in SC constructors.
*/
#define A_KRIG_APPROX 1
#define A_NULL_APPROX 2

class Approximate {

public:
  Approximate();
  Approximate(Approximate &A);
  Approximate(long dim);
  /*call AutoGenerate*/
  virtual ~Approximate();
  virtual Approximate& operator=(Approximate &A)=0;

  /*****RESET FUNCTIONS*****/
  void SetDim(long dim);
  /****
  INPUT: Dimension of the space.
  EFFECTS: Sets the problem dimension to dim.
  ****/
  
  /*****QUERY FUNCTIONS*****/

  long GetDim() const;
  /****
  OUTPUT: Returns the dimension of the space.
  ****/ 

  virtual long GetType() const=0;
  /****
  OUTPUT: returns the proper code for type of approximation
  ****/     
  
  virtual void debug(char *code) const=0;
  /****
  EFFECT: Outputs a state dump.
  ****/

  /*****ACTION FUNCTIONS*****/
  
  /* These functions are purely virtual. Look at KrigApprox and
     NullApprox for implementation */
  
  virtual bool EvaluateApprox(const mp_vector &x, const pt_collect &Eval_c, double* Approx,
                              double* D_c)=0;
  /****
  INPUT: The location to evaluate at, the location of evaluated sites, and two
  double pointers for return values.
  OUTPUT: Returns true if the operation was successful.
  EFFECT: It Approximates, and returns the Approximation and Design Criterion
  Values in the two pointers
  ****/
  
  virtual bool UpdateApprox(pt_collect &Eval_c, mp_vector &f_values, long
                            numpoints, double delta,const Bounds &b)=0;
  /****
  INPUT: The collection of evaluated sites, their values, the number of points
  evaluated, delta and the bounds.
  OUTPUT: Returns true if the operation was sucessful.
  EFFECT: This updates the approximation.
  ****/
protected:
  long P;         /*Dimension of the space*/
  

};/*end class*/

#endif
