/****
InitialDesign Class Implementation- 6/19/00 cmsief
This is the parent class for Initial Designs.  Just inherit this, and you're
almost all set.
****/

#include "InitialDesign.h"

InitialDesign::InitialDesign() {
  init_done=false;
  P=0;
  N=0;
  points=NULL;
}/*end default constructor*/

InitialDesign::InitialDesign(long dim){
  init_done=false;
  P=dim;
  N=0;
  points=NULL;
}/*end special constructor*/


InitialDesign::InitialDesign(const InitialDesign &I) {
  init_done=I.init_done;
  P=I.P;
  N=I.N;
  if(I.points!=NULL) points=new pt_collect(*I.points); 
  else points=NULL;
}/*end copy constructor*/


InitialDesign::~InitialDesign() {
  if(points!=NULL) delete points;
}/*end destructor*/

InitialDesign& InitialDesign::operator=(const InitialDesign &I) {
  if(this!=&I) {
    init_done=I.init_done;
    P=I.P;
    N=I.N;
    if(points!=NULL) delete points;
    if(I.points!=NULL) points=new pt_collect(*I.points);
    else points=NULL;
  }/*end if*/
  return (*this);
}/*end overloaded =*/

void InitialDesign::SetDimension(long dim) {
  P=dim;
}/*end SetDimension*/


bool InitialDesign::GenerateDesign(const Bounds &b, long num_points){
/****
INPUT: The problem's bounds and the number of points to put in the design.
This assumes, of course that you've set up rngs right.
OUTPUT: Returns true.
EFFECTS: Sets init_done=true
Derived classes must call this function. 
****/ 
  init_done=true;
  return true;
}/*end GenerateDesign*/


void InitialDesign::GetDesign(pt_collect* outptr){
/****
INPUT: Pointer of type pt_collect*.
EFFECTS: Copies points to outptr if init_done==true.
****/
  (*outptr)=(*points);
}/*end GetDesign*/


void InitialDesign::debug(char *code) {
  cout<<code<<" InitialDesign::debug()\n";
  cout<<code<<" P="<<P<<" N="<<N<<endl;
  cout<<code<<(*points);

}/*end debug*/

bool InitialDesign::MoveToGrid(const Bounds &b, const PSGrid &grid) {
/****
INPUT: The problem's bounds and grid.
EFFECTS: Moves all the points in 'points' to grid position.  Calls
grid.SnapToGrid, and uses that to do the snapping.  Then it just fixes the
point list.
****/

    if (init_done) {
        mp_vector temp(P);
        
        for(long i=0; i<N;i++){
            grid.SnapToGrid(b,(*points).row(i),&temp);
            for(long j=0;j<P;j++)
                ((*points)[i])[j]=temp[j];
        }/*end for*/
        return true;
    }
    else {
        cout << "Design must be generated before it can be moved\n";
        return false;
    }
}/*end MovetoGrid*/





