/****
InitialDesign Class - 6/19/0 cmsief
This is the parent class from which all InitialDesign derived classes shall be
derived.  Latin Hypercube design is the sample derived class.
****/

#ifndef _MAPS_ID_
#define _MAPS_ID_

#include "maps_general.h"
#include "PSGrid.h"

class InitialDesign {

public:
  InitialDesign(); /*default constructor*/
  /*init_done=false*/
  InitialDesign(long dim);  /*sets the dimension of the space*/
  /*init_done=false*/
  InitialDesign(const InitialDesign &I); 
  virtual ~InitialDesign();
  virtual InitialDesign& operator=(const InitialDesign &I);
  
  void SetDimension(long dim);
  /****
  INPUT: The dimension of the problem
  EFFECTS: P=dim;
  ****/
  
  virtual bool GenerateDesign(const Bounds &b, long num_points);
  /****
   INPUT: The problem's bounds and the number of points to put in the design.
   This assumes, of course that you've set up rngs right.
   OUTPUT: Returns true if the operation was successful.
   EFFECTS: Sets init_done=true
   Derived classes must call this..   
  ****/ 

  void GetDesign(pt_collect* outptr);
  /****
  INPUT: Pointer of type pt_collect*.
  EFFECTS: Copies points to outptr if init_done==true.
  ****/

  long GetDesignSize() {return N;}
  /****
  OUTPUT: returns N.
  ****/
  
  bool IsDesignInitialized()const {return init_done;}
  /****
  OUTPUT: Returns true iff design is done.
  ****/

  bool MoveToGrid(const Bounds &b, const PSGrid &grid);
  /****
   INPUT: The problem's bounds and grid.
   EFFECTS: Moves all the points in 'points' to grid position.  Calls
   grid.SnapToGrid, and uses that to do the snapping.  Then it just fixes the
   point list. Returns true if successful.
  ****/

  virtual void debug(char *code);
  /****
  EFFECT: Outputs a state dump of the object.
  ****/
  
protected:
  long P; /*dimension*/
  long N; /*number of points for design*/
  pt_collect* points;
  
private:
 
  bool init_done;
  
};/*end class*/

#endif

