/****
LatinHCDesign Class Implementation - 6/19/00 cmsief
Derived from InitialDesign
****/
#include "LatinHCDesign.h"

LatinHCDesign::LatinHCDesign():InitialDesign(){stream=0;}
/*default constructor*/

LatinHCDesign::LatinHCDesign(long dim):InitialDesign(dim){stream=0;}
/*special constructor*/

LatinHCDesign::LatinHCDesign(const LatinHCDesign &I):InitialDesign(I){
    stream=0;
}
/*copy constructor*/

LatinHCDesign::~LatinHCDesign(){;}
/*destructor*/

bool LatinHCDesign::GenerateDesign(const Bounds &b, long num_points) {
/****
INPUT: The problem's bounds and the number of points to put in the design
EFFECTS: Generates the initial design via latin hypercubes.  Then calls
InitialDesign's GenerateDesign.
****/
  N=num_points;
  ml_vector a(num_points);/*Partition of a given dimension*/
  long s,temp;
  double int_width;/*width of interval*/
  long i,j; /*counters*/

  if(points!=NULL) delete points;
  points=new pt_collect(N,P);
  
  /* Select the random stream - Calls method in rngs.c */
  SelectStream(stream);  

  for(i=0;i<P;i++) { /*loop on dimension*/
      int_width = (b.upper[i]-b.lower[i])/N;    
      for(j=0;j<N;j++) a[j]=j;/*clear array*/
      
      /*Shuffle the array*/
      for(j=0;j<N;j++) {
          s=Equilikely(j,N-1);
          temp=a[s];
          a[s]=a[j];
          a[j]=temp;
      }/*end for*/
      /*END SHUFFLE*/

      /*Fill array 'points' with uniformly spaced points inside bounds*/
      for(j=0;j<N;j++)
          ((*points)[j])[i]=
              b.lower[i]+Uniform(a[j]*int_width,(a[j]+1)*int_width);
  }/*end for dimension*/
  
  /*Just sets init_done to true*/
  InitialDesign::GenerateDesign(b,num_points);
  return true;
}/*end GenerateDesign*/

void LatinHCDesign::PutStream(long st) {
    stream = st;
}


