/****
SearchCriterion implementation - 6/27/00 cmsief
****/
#include <iostream>
#include "maps_general.h"
#include "Approximate.h" 
#include "SearchCriterion.h"
#include "KrigApprox.h"
#include "NullApprox.h"

/*weighting schemes*/
//#define SC_WEIGHT_MULTIPLE 0
//#define SC_WEIGHT_TARGET_VALUE 1
//#define SC_WEIGHT_DELTA_MULTIPLE 2
//#define SC_WEIGHT_DYNAMIC_GUESS 3
//#define SC_WEIGHT_DYNAMIC 4

SearchCriterion::SearchCriterion() {
  w_scheme=ERROR;
  w_params=NULL;
  np=0;
  APX=NULL;
  Eval_c=NULL;f_values=NULL;
  w_c=ERROR;
  //P=0;
  original_delta=ERROR;first_time=true;
}/*end default constructor*/

SearchCriterion::SearchCriterion(long dim,Bounds &b) {
  bds=&b;first_time=true;
  SetDefault(dim);
}/*end default constructor*/
  
SearchCriterion::SearchCriterion(const SearchCriterion &S) {
  w_scheme=S.w_scheme;
  w_params=new mp_vector(*S.w_params);
  np=S.np;
  APX=NULL;
  ResetApproximation(*S.APX);
  Eval_c=S.Eval_c;     //this is on purpose.
  f_values=S.f_values; //so is this.
  w_c=S.w_c;
  //P=S.P;
  bds=S.bds;
  original_delta=S.original_delta;first_time=S.first_time;
}/*end copy constructor*/
  
SearchCriterion::SearchCriterion(Approximate &A,Bounds &b) {
    //P=A.GetDim();
  w_scheme=SC_WEIGHT_DYNAMIC;
  w_params=new mp_vector(3);
  (*w_params)[0]=2;/*default value*/
  (*w_params)[1]=1.25;/*default value*/
  (*w_params)[2]=0.75;/*default value*/
  bds=&b;
  np=0;w_c=ERROR;
  Eval_c=NULL;f_values=NULL;
  APX=NULL;first_time=true;
  original_delta=ERROR;
  ResetApproximation(A);
}/*end special constructor*/

SearchCriterion::~SearchCriterion(){
  if(w_params!=NULL) delete w_params;
  /*Don't delete Eval_c.  Trust me on this one*/
  if(APX!=NULL) delete APX;
}/*end destructor*/

SearchCriterion& SearchCriterion::operator=(SearchCriterion &S){
  if(this !=&S) {
    bds=S.bds;
    w_scheme=S.w_scheme;
    (*w_params)=(*S.w_params);
    np=S.np;
    ResetApproximation(*S.APX);
    (*Eval_c)=(*S.Eval_c);
    (*f_values)=(*S.f_values);    
    w_c=S.w_c;
    //P=S.P;
    original_delta=S.original_delta;
    first_time=S.first_time;
  }/*end if*/
  return (*this);
}/*end overloaded =*/

bool SearchCriterion::ResetSearchCriterionState(Approximate &A, int scheme_number, mp_vector
                               &params){
/****
INPUT: An Approximation, a weight scheme number and its parameters.
OUTPUT: returns true iff ResetWeightScheme returns true.
EFFECT: Calls ResetApproximation, and ResetWeightScheme.
****/
  ResetApproximation(A);
  return ResetWeightScheme(scheme_number, params);
}/*end ResetWeightScheme*/


void SearchCriterion::ResetApproximation(Approximate &A) {
/****
INPUT: An Approximation.
EFFECTS: (*APX) = A
****/
  if(APX!=&A) {
    if(APX!=NULL) delete APX;
    switch(A.GetType()) {
    case A_NULL_APPROX:    
      APX=new NullApprox();
      (*APX)=(NullApprox&)A;
      break;
    case A_KRIG_APPROX:
      APX=new KrigApprox();
      (*APX)=(KrigApprox&)A;
      break;
    default:
#if DEBUG>=1
      cerr<<"SCRA   :Approximation Type Incorrect\n";
#endif
      break;
    };/*end case*/
  }/*end if*/
}/*end ResetApproximation*/

bool SearchCriterion::ResetWeightScheme(int scheme_number, mp_vector &params) {
/****
INPUT: The scheme number and its parameters.
OUTPUT: Returns true if everything is valid.
EFFECTS: This sets the w_scheme integer, and then sets w_params.  This will
also set w_c to the correct initial value
****/
  bool success;
  
  switch(scheme_number){
  case SC_WEIGHT_MULTIPLE:
    if(params.dim() !=2) return false;/*sanity check*/
    w_scheme=SC_WEIGHT_MULTIPLE;
    if(w_params==NULL) w_params=new mp_vector(params);
    else {(*w_params).newsize(2); (*w_params)=params;}
    success=true;
    break;
  case SC_WEIGHT_TARGET_VALUE:
    if(params.dim()!=3) return false;/*sanity check*/
    w_scheme=SC_WEIGHT_TARGET_VALUE;
    if(w_params==NULL) w_params=new mp_vector(4);
    else (*w_params).newsize(4);
    CreateWeightTargetValue(params);
    success=true;
    break;    
  case SC_WEIGHT_DELTA_MULTIPLE:
    if(params.dim()!=2) return false;/*sanity check*/
    w_scheme=SC_WEIGHT_DELTA_MULTIPLE;
    if(w_params==NULL) w_params=new mp_vector(2);
    else (*w_params).newsize(2);
    (*w_params)=params;
    w_c=(*w_params)[0];
    success=true;
    break;
  case SC_WEIGHT_DYNAMIC_GUESS:
    if(params.dim() !=4) return false;/*sanity check*/
    w_scheme=SC_WEIGHT_DYNAMIC_GUESS;
    if(w_params==NULL) w_params=new mp_vector(params);
    else {(*w_params).newsize(4); (*w_params)=params;}
    w_c=(*w_params)[0];
    success=true;
    break;    
  case SC_WEIGHT_DYNAMIC:
    if(params.dim() !=3) return false;/*sanity check*/
    w_scheme=SC_WEIGHT_DYNAMIC;
    if(w_params==NULL) w_params=new mp_vector(params);
    else {(*w_params).newsize(3); (*w_params)=params;}
    w_c=(*w_params)[0];
    success=true;
    break;    
  default:
    success=false;
  };/*end case*/
  return success;  
}/*end SetWeightScheme*/


void SearchCriterion::EvaluateSearchCriterion(long dim, mp_vector &x, double &f, bool &success) {
/****
INPUT: The dimension of the space, a list of values, a reference passed value
of the function, and a reference-passed flag.  The input syntax is this
convoluted to ensure that it works with Anne Shepherd's version of Liz Dolan's
PatternSearch.
EFFECT: This evaluates the search criterion at a given point, calling
Approximate::Evaluate to do so, and takes the weighting schedule into
account.  If the evaluation was a success, then success is 1, else it is
set to 0.
****/
  for(long i=0;i<dim;i++) 
    if(x[i]<(*bds).lower[i] || x[i]> (*bds).upper[i]) {
         success=false; 
#if DEBUG>=3
  cout<<"SCESC  :POINT OUT OF BOUNDS\n";
#endif 
         return;
    }/*end if*/
  
  double f_true=ERROR,fhat_c, d_c;
  bool realpoint=false;
  mp_vector temp=x;

  /*This check should prevent degradation at previously eval'd sites*/
  for(long j=0;!realpoint&&j<(*Eval_c).num_rows();j++) {
    if(isnear((*Eval_c).row(j),temp,TOLERANCE)) {
      f_true=(*f_values)[j];/*set approx to true value*/
      realpoint=true;
    }/*end if*/
  }/*end for*/
  
  success=(*APX).EvaluateApprox(temp, (*Eval_c), &fhat_c, &d_c);

  if(realpoint) fhat_c=f_true; /*for degradation check*/

#if DEBUG>=3
  cout<<"SCESC  :hat{f_c}="<<fhat_c<<" w_c="<<w_c<<" d_c="<<d_c<<endl;
#endif
  if(success) f=(fhat_c - w_c * d_c);
  else f=ERROR;

}/*end EvaluateSearchCriterion*/

void SearchCriterion::GetApproximationGraphics(long dim, mp_vector &x, double &f, bool &success){
/****
INPUT: PatternSearch calling sequence.  See PatternSearch.h
OUTPUT: Value of the Approximation only at that point.  Meant for use by
GenerateMAPSGraphics.
****/
  for(long i=0;i<dim;i++) 
    if(x[i]<(*bds).lower[i] || x[i]> (*bds).upper[i]) {
      success=false; 
      return;
    }/*end if*/
  
  double fhat_c, d_c;
  mp_vector temp=x;  
  success=(*APX).EvaluateApprox(temp, (*Eval_c), &fhat_c, &d_c);
  if(success) f=fhat_c;
  else f=ERROR;
}/*end GetApproximationGraphics*/

bool SearchCriterion::UpdateSearchCriterion(pt_collect &points, mp_vector& values, long
                                            numpoints, long n, double delta){
/****
INPUT: The list of evaluated sites, the values, and the number of points
listed, the number of points in the initial design, and the curent delta.
OUTPUT: Returns true if the operation was sucessful.
EFFECT: This will call (*APX).Update to update the approximation, then
will update the weighting scheme by calling the appropriate UpdateWeight function.
****/
  if((points.num_rows() != numpoints)||(values.dim()!=numpoints)) {
    cerr<<"SCUSC  :ERROR!  Number of Points doesn't match parameters"<<endl;
  }/*end if*/
    
  Eval_c=&points;/*I know this is a bad idea.*/
  f_values=&values;/*so is this - but this is for memory conservation*/
  np=numpoints;  
  if(original_delta==ERROR) original_delta=delta;
  
#if DEBUG>=3
  cout<<"SCUSC  :Old weight="<<w_c;
#endif
  if(first_time) first_time=false;
  else{
    switch(w_scheme) {
    case SC_WEIGHT_MULTIPLE:
      UpdateWeightMultiple(numpoints-n);
      break;
    case SC_WEIGHT_TARGET_VALUE:
      UpdateWeightTargetValue(numpoints-n);
      break;
    case SC_WEIGHT_DELTA_MULTIPLE:
      UpdateWeightDeltaMultiple(delta);    
      break;
    case SC_WEIGHT_DYNAMIC_GUESS:
      UpdateWeightDynamicGuess(numpoints,values);
      break;
    case SC_WEIGHT_DYNAMIC:
      UpdateWeightDynamic(numpoints,values);
      break;
    default:
      break;
    };/*end switch*/
  }/*end else*/
#if DEBUG>=3
  cout<<", New weight="<<w_c<<endl;
#endif

  bool success=(*APX).UpdateApprox(points, values, numpoints,delta, *bds);

  return success;  
}/*end UpdateSearchCriterion*/


void SearchCriterion::debug(char *code) const {
/****
EFFECT: does a complete state dump to stdout.
****/ 
  cout<<code<<":***SearchCriterion::debug()***\n";
  //cout<<code<<":P="<<P<<" np="<<np<<endl;
  cout<<code<<":w_scheme="<<w_scheme<<" w_c="<<w_c<<" w_params="<<(*w_params);

  (*APX).debug(code);
  cout<<code<<":***end SC:debug()***\n";
}/*end debug*/


void SearchCriterion::SetDefault(long dim){
/****
INPUT: Dimension of space
EFFECT: Automagically generates one of these objects, uses (2,1.25,0.75)
WeightDynamic scheme and Kriging Approximation.
****/
    //P=dim;
  w_scheme=SC_WEIGHT_DYNAMIC;
  w_params=new mp_vector(3);
  (*w_params)[0]=2;   /*default value*/
  (*w_params)[1]=1.25;/*default value*/
  (*w_params)[2]=0.75;/*default value*/
  np=0;w_c=0;
  Eval_c=NULL;
  APX=new KrigApprox(dim);/*defaults to Kriging*/  
}/*end SetDefault*/

  
  /*Weighting Scheme functions*/
void SearchCriterion::CreateWeightTargetValue(mp_vector &params){
/*This does the computation of w_params[3]
  m = (p[1] / p[0])^(1/(p[2] * (V-N)))
*/
  for(long i=0;i<3;i++) (*w_params)[i]=params[i];
  (*w_params)[3]= pow( params[1] / params[0], 1.0 / params[2]);  
}/*end CreateWeightTargetValue*/

void SearchCriterion::UpdateWeightTargetValue(long c){
/****
INPUT: The iteration number.
EFFECT: Multiply w_c by the correct multiplier in order to reach the level of
insignificance by the specified iteration.
PARAMS FORMAT: [0] - w_0
               [1] - target value of insignificance
               [2] - iteration number at which to reach insignificance
Though w_params[3] is not specified by the user, when you install the Target
Value weighting scheme, the multiple is computed and stored here.
****/
  w_c=(*w_params)[0];
  for(long i=0;i<c;i++) w_c *= (*w_params)[3];  
}/*end UpdateWeightTargetValue*/


void SearchCriterion::UpdateWeightMultiple(long c) {
/****
EFFECTS: This will multiply w_c by w_params[1]
PARAMS FORMAT: [0] - w_0
               [1] - c, s.t. w_c = c * w_{c-1}
****/
  w_c=(*w_params)[0];
  for(long i=0;i<c;i++) w_c *= (*w_params)[1];  
}/*end UpdateWeightMultiple*/


void SearchCriterion::UpdateWeightDeltaMultiple(double curr_delta) {
/****
INPUT: current delta
EFFECT: If delta has been refined, the weight is multiplied by w_1
PARAMS FORMAT: [0] - w_0, the weight before delta refinement
               [1] - m, s.t. w_c = m*w_{c-1}, if delta was refined last iteration
                                   = w_{c-1}, if delta wasn't refined last iteration
****/
  if(original_delta-curr_delta>=TOLERANCE) {
    original_delta=curr_delta;
    w_c *= (*w_params)[1];
  }/*end if*/
}/*end UpdateDeltaBased*/


void SearchCriterion::UpdateWeightDynamicGuess(long lastsite, mp_vector& values){
/****
INPUT: number of last site eval'd and vector of function values
EFFECT: If the approximation was 'close' to correct at the last site, then we
reduce the weight, less we increase it.
PARAMS FORMAT: [0] - w_0
               [1] - t, the threshhold percentage
               [2] - m_b, multiplier if approximation is 'bad'
               [3] - m_g, multiplier if approximation is 'good'
               
               w_c =  m_g * w_{c-1}, if |f(x_{c-1}) -\hat{f_{c-1}(x_c-1)}| <= t*frange
                   =  m_b * w_{c-1}, otherwise
****/
  double v_min,v_max;
  double f_hat, d_c;
  bool s;
#if DEBUG>=2
  cout<<"LASTSITE="<<lastsite;
#endif
  mp_vector x(1);
  x=(*Eval_c).row(lastsite-1);
  
  /*Compute the Function range*/
  v_min=v_max=values[0];
  for(long i=1;i<lastsite;i++) {
    if(values[i]<v_min) v_min=values[i];
    if(values[i]>v_max) v_max=values[i];    
  }/*end for*/

  /*Eval Old Approximation*/
  s=(*APX).EvaluateApprox(x, *Eval_c, &f_hat,&d_c);

  if(fabs(values[lastsite-1] - f_hat) <= (*w_params)[1] *(v_max - v_min)){
    w_c *= (*w_params)[3];/*the approx is good*/
#if DEBUG>=2
    cout<<"... approx is good ";
#endif
  }
  else {
    w_c *= (*w_params)[2];/*approx is not so good*/  
#if DEBUG>=2
    cout<<"... approx is not good ";
#endif
  }
#if DEBUG>=2  
  cout<<"w_c="<<w_c<<endl;
#endif
  
}/*end UpdateWeightDynamicGuess*/


void SearchCriterion::UpdateWeightDynamic(long lastsite, mp_vector& values){
/****
INPUT: number of last site eval'd and vector of function values
EFFECT: If the approximation was 'close' to correct at the last site, then we
reduce the weight, less we increase it.  Similar to DynamicGuess, except it uses 
|expval - bestpoint| / 2 as the threshold. 
PARAMS FORMAT: [0] - w_0
               [1] - m_b, multiplier if approximation is 'bad'
               [2] - m_g, multiplier if approximation is 'good'
                 
               w_c =  m_g * w_{c-1}, if |f(x_{c-1}) -\hat{f_{c-1}(x_c-1)}| <= |expval-best|/2
                   =  m_b * w_{c-1}, otherwise
****/
  double v_min;
  double f_hat, d_c;
  bool s;
#if DEBUG>=2
  cout<<"LASTSITE="<<lastsite;
#endif
  mp_vector x(1);
  x=(*Eval_c).row(lastsite-1);
  
  /*Compute the Function range*/
  v_min=values[0];
  for(long i=1;i<lastsite-1;i++) { //avoid the last site for this calulation - get old min
    if(values[i]<v_min) v_min=values[i];
  }/*end for*/

  /*Eval Old Approximation*/
  s=(*APX).EvaluateApprox(x, *Eval_c, &f_hat,&d_c);

  if(fabs(values[lastsite-1] - f_hat) <= fabs((v_min - f_hat)/2.0)){
    w_c *= (*w_params)[2];/*the approx is good*/
#if DEBUG>=2
    cout<<"... approx is good ";
#endif
  }
  else {
    w_c *= (*w_params)[1];/*approx is not so good*/  
#if DEBUG>=2
    cout<<"... approx is not good ";
#endif
  }
#if DEBUG>=2  
  cout<<"w_c="<<w_c<<endl;
#endif
  
}/*end WeightDynamicBest*/





