#include <iostream>
#include <cstdio>
#include <cstring>
#include "maps_general.h"
#include "maps.h"
#include "PSGrid.h"
#include "InitialDesign.h"
#include "SearchCriterion.h"
#include "Approximate.h"
#include "KrigApprox.h"
#include "NullApprox.h"
#include "malloc.h"
#include "rngs.h"
#include "rvgs.h"
#include "ParamEstimate.h"
#include "CompassSearch.h"
#include "MultiCompassSearch.h"

#define MAX_STRING_SIZE 108    //this is probably the execlp limit anyway

//#define COR_EXP_ISOTROPIC 0
//#define COR_GAUSS_ISOTROPIC 1
//#define COR_EXP_PRODUCT 2
//#define COR_GAUSS_PRODUCT 3
//#define COR_CUBIC_ISOTROPIC 4

#define DEFAULT -1

long R_SEED;  /*random seed*/
long DIM;
long RUNS;
long V;
long N;
long WEIGHT;
double delta;
long COR;
long PS;
long trend;
char fn[MAX_STRING_SIZE];
char it_out[MAX_STRING_SIZE];
mp_vector wparams(1);
ml_vector MCSparams(3);
Bounds b;

/*"Robust" Input methods*/

char buff[256];
long bsize=255;
int redo=0;
FILE * fs=stdin;

int pscan_line();
int pscan_long(long* rv);
int pscan_double(double* rv);
int pscan_string(char* rv);

using namespace std;

void GetParameters() {
  long i;
  cout<<"Welcome to the MAPS test program!\n";
  cout<<"Please enter the following information:\n";
  cout<<"Random Seed (integer)=";pscan_long(&R_SEED);
  cout<<"Number of Runs of MAPS=";pscan_long(&RUNS);  
  cout<<"Number of Variables=";pscan_long(&DIM);
  cout<<"Total Budget[V]=";pscan_long(&V);
  cout<<"Initial Design Budget[N] (less than total)=";pscan_long(&N);
  cout<<"Initial Mesh Size[Delta]=";pscan_double(&delta);//this generates a vector with the same resolution;
  cout<<"Correlation Family(default=-1,EI=0,GI=1,EP=2,GP=3,CI=4)=";pscan_long(&COR);
  if(COR <=-1 || COR > 4) {cout<<"Using default correlation...\n"; COR=DEFAULT;}
  cout<<"Pattern Search(default=-1,Compass=0, MultiCompass=1)=";pscan_long(&PS);
  if(PS <=-1 || PS > 1) {cout<<"Using default pattern search...\n"; PS=DEFAULT;}
  else if(PS==1) {
    cout<<"MultiCompass Params(#searches, maxcalls, stream)=";
    pscan_long(&MCSparams[0]);pscan_long(&MCSparams[1]);pscan_long(&MCSparams[2]);
  }/*end if*/
  cout<<"Assumed Trend (default=-1, Constant=0, Quadratic=1)=";pscan_long(&trend);
  if(trend <=-1 || trend > 1) {cout<<"Using default trend...\n"; trend=0;}
  cout<<"Weighting Scheme(default=-1,WM=0,WTV=1,WDM=2,WDG=3,WD=4)=";pscan_long(&WEIGHT);
  if(WEIGHT==SC_WEIGHT_MULTIPLE) {
    wparams.newsize(2);
    cout<<"Weighting Params (w_0, m)=";pscan_double(&wparams[0]);pscan_double(&wparams[1]);
  }/*end if*/
  else if(WEIGHT==SC_WEIGHT_TARGET_VALUE){
    wparams.newsize(3);
    cout<<"Weighting Params (w_0,w_i,i%)=";pscan_double(&wparams[0]);
    pscan_double(&wparams[1]);pscan_double(&wparams[2]);
    if(wparams[2] > 1) wparams[2] *= 0.01; /*convert to percentile*/
    wparams[2] *= (V-N); /*get it to compute insignifcant iteration*/
  }/*end else if*/
  else if(WEIGHT==SC_WEIGHT_DELTA_MULTIPLE){
    wparams.newsize(2);
    cout<<"Weighting Params (w_0,w_1)=";pscan_double(&wparams[0]);pscan_double(&wparams[1]);
  }/*end else if*/
  else if(WEIGHT==SC_WEIGHT_DYNAMIC_GUESS){
    wparams.newsize(4);
    cout<<"Weighting Params (w_0,t%,m_b,m_g)=";
    pscan_double(&wparams[0]);pscan_double(&wparams[1]);
    pscan_double(&wparams[2]);pscan_double(&wparams[3]);
  }/*end else if*/   
  else if(WEIGHT==SC_WEIGHT_DYNAMIC){
    wparams.newsize(3);
    cout<<"Weighting Params (w_0,m_b,m_g)=";pscan_double(&wparams[0]);
    pscan_double(&wparams[1]);pscan_double(&wparams[2]);
  }/*end else if*/   
  else {cout<<"Using default weights...\n"; WEIGHT=DEFAULT;}
  
  cout<<"Program name of function to optimize="; 
  pscan_string(fn);
  cout<<"If MAPS crashes, try the full path name of the program instead\n";
  
  b.lower.newsize(DIM);b.upper.newsize(DIM);
  cout<<"Lower bound (space separated)=";
  for(i=0;i<DIM;i++) pscan_double(&b.lower[i]);
  cout<<"Upper bound (space separated)=";
  for(i=0;i<DIM;i++) pscan_double(&b.upper[i]);
}/*end GetParameters*/


void RunTest(bool gfx, bool iteration) {
  bool s1,s2,s3,s4,s5;
  mp_vector res(DIM);
  res=delta;//assign to all elements
  PSGrid g(DIM,b,res);

  LatinHCDesign LHC1(DIM);
  // You can change the random stream before you generate a design
  // (LHC Only)
  LHC1.PutStream(0);
  // This generates the Latin Hypercube.
  LHC1.GenerateDesign(b,N);
  // If you want to move the initial design onto the PS grid
  LHC1.MoveToGrid(b,g);
   
  maps M(V,DIM,b,fn);

  /* Constructor calls SetDefault to initialize ParamEstimate, v, theta,
  etc. */
  KrigApprox KA(DIM);

  /* Change defaults if necessary */
  if(COR!=DEFAULT)
    KA.ResetCorrelationFamily(COR);  
  if(trend==1)
    KA.ResetEstimateQuadraticTrend();

  /* SearchCriterion must know approximation and bounds */
  SearchCriterion SC(KA,b);

  mp_vector temp(DIM);temp=0;
  CompassSearch CS(DIM,temp);
  MultiCompassSearch MCS(DIM,temp);    
  if(PS==1)
    MCS.ResetParams(MCSparams[0],MCSparams[1],MCSparams[2],b);
  
  if(WEIGHT!=DEFAULT)
    SC.ResetWeightScheme(WEIGHT, wparams);
  mp_vector bestpt(DIM);
  double bestval;

 
  if(PS==0) s1=M.ResetPatternSearch(CS);
  else if(PS==1) s1=M.ResetPatternSearch(MCS);
    
  s2=M.ResetDesign(LHC1);
  s3=M.ResetSearchCriterion(SC);
  s4=M.ResetGrid(g);

  if(iteration) s5=M.RunMAPS(it_out);
  else s5=M.RunMAPS();
  M.RetrieveBestSeen(&bestpt,&bestval);
    M.GenerateIterationHistory();
  //printf("point ");
  //for(long i=0;i<DIM;i++) printf("%1.8f ",bestpt[i]);
  //printf("   value %1.8f\n",bestval);
  cout<<"Bestval="<<setw(10)<<bestval<<endl;
  
  if (gfx==true) M.GenerateMAPSGraphics(fn);
}/*end RunTest*/

int main(int argc, char*argv[]) {
  bool gfx=false, iteration=false;

  if(argc==2 && !strcmp(argv[1],"--graphics")) gfx=true;
  if(argc==3 && !strcmp(argv[1],"--iteration")) {
    iteration=true;
    strcpy(it_out,argv[2]);
  }/*end if*/
  else if(argc==2 && !strcmp(argv[1],"--help")) {
    cout<<argv[0]<<" [--graphics || --help || {--iteration <fpref>}]\n"
        <<"Welcome to the MAPS test program!\n"
        <<"Chris Siefert, College of William and Mary 2000\n";
    return 0; 
  }/*end if*/

  GetParameters();

  cout<<"**********"<<endl;
  cout<<"fn="<<fn<<" V="<<V<<" N="<<N<<" delta="<<delta<<endl;
  PlantSeeds(R_SEED);
  GetSeed(&R_SEED);//in case user feeds in -1
  cout<<"Random seed="<<R_SEED<<endl;
  
  for(long i=0;i<RUNS; i++) {
    RunTest(gfx,iteration);
  }/*end for*/
  
  
  return 0;
}/*end main*/

int pscan_line(){
  char* idx,*s;
  do{
    s=fgets(buff, bsize, fs);
    /*Remove comments*/
    if (s!=NULL && (idx=strchr(buff,'#'))!=NULL)
      *idx='\0';
  }while(s!=NULL && strlen(buff)==0);
  return s!=NULL;
}/*end pscan_line*/



int pscan_long(long* rv){
  int rv1=1,rv2=0;
  char * tmp;
  if(redo) {rv2=sscanf(buff,"%ld",rv);redo=0;}
  while( rv1 && rv2<1){
    rv1=pscan_line();
    if(rv1) rv2=sscanf(buff,"%ld",rv);    
  }/*end while*/
  if(rv2>0) {
    tmp=strchr(buff,' ');
    if(tmp!=NULL){
      while(*tmp==' ') tmp++;
      memmove(buff,tmp, 127-(tmp-buff)*sizeof(char));redo=1;
      }/*end if*/
  }/*end if*/
  return rv2>0;  
}/*pscan_long*/

int pscan_double(double* rv){
  int rv1=1,rv2=0;
  char * tmp;
  if(redo) {rv2=sscanf(buff,"%lf",rv);redo=0;}
  while( rv1 && rv2<1){
    rv1=pscan_line();
    if(rv1) rv2=sscanf(buff,"%lf",rv);    
  }/*end while*/
  if(rv2>0) {
    tmp=strchr(buff,' ');
    if(tmp!=NULL){
      while(*tmp==' ') tmp++;
      memmove(buff,tmp, 127-(tmp-buff)*sizeof(char));redo=1;
      }/*end if*/
  }/*end if*/
  return rv2>0;  
}/*pscan_double*/

int pscan_string(char* rv){
  int rv1=1,rv2=0;
  char * tmp;
  if(redo) {rv2=sscanf(buff,"%s",rv);redo=0;}
  while( rv1 && rv2<1){
    rv1=pscan_line();
    if(rv1) rv2=sscanf(buff,"%s",rv);    
  }/*end while*/
  if(rv2>0) {
    tmp=strchr(buff,' ');
    if(tmp!=NULL){
      while(*tmp==' ') tmp++;
      memmove(buff,tmp, 127-(tmp-buff)*sizeof(char));redo=1;
      }/*end if*/
  }/*end if*/
  return rv2>0;
}/*end if*/
